import { Ref, UnwrapRef, ComputedRef, StopHandle } from '@pksilen/reactive-js';
import { SubStateFlagWrapper } from './createSubState';
import { Writable } from 'svelte/store';
export declare type SubState = Omit<object, '__isSubState__'> & SubStateFlagWrapper;
export declare type State = {
    [key: string]: SubState;
};
export declare type SelectorsBase<T extends State> = {
    [key: string]: (state: T) => any;
};
export declare type Selectors<T extends State, U extends SelectorsBase<T>> = {
    [K in keyof U]: (state: T) => ReturnType<U[K]>;
};
declare type ComputedSelectors<T extends State, U extends SelectorsBase<T>> = {
    [K in keyof U]: ComputedRef<ReturnType<U[K]>>;
};
declare type ReactiveState<T> = T extends Ref ? T : UnwrapRef<T>;
export default class Store<T extends State, U extends SelectorsBase<T>> {
    private readonly reactiveState;
    private readonly reactiveSelectors;
    private readonly stateStopWatches;
    private readonly selectorStopWatches;
    private viewToNeedsUpdateMap;
    private readonly componentInstanceToUpdatesMap;
    private readonly stateWritables;
    private readonly selectorWritables;
    private readonly idToUpdatesMap;
    constructor(initialState: T, selectors?: Selectors<T, U>);
    getState(): ReactiveState<T>;
    getSelectors(): ComputedSelectors<T, U>;
    getStateAndSelectors(): [ReactiveState<T>, ComputedSelectors<T, U>];
    useStateAndSelectorsReact(subStates: SubState[], selectors: ComputedRef[]): void;
    watchSubStatesAndSelectors(subStates: SubState[] | ComputedRef[], stopWatches: StopHandle[], view: {}, updateView: (newState: {}) => void): void;
    useStateReact(subStates: SubState[]): void;
    useSelectorsReact(selectors: ComputedRef[]): void;
    useStateAndSelectorsNg<V extends new (...args: any[]) => any>(componentInstance: InstanceType<V>, subStateMap: {
        [key: string]: SubState;
    }, selectorMap: {
        [key: string]: ComputedRef<any>;
    }): void;
    useStateNg<V extends new (...args: any[]) => any>(componentInstance: InstanceType<V>, subStateMap: {
        [key: string]: SubState;
    }): void;
    useSelectorsNg<V extends new (...args: any) => any>(componentInstance: InstanceType<V>, selectorMap: {
        [key: string]: ComputedRef<any>;
    }): void;
    useStateSvelte(id: string, subStates: SubState[]): Writable<SubState>[];
    useSelectorsSvelte(id: string, selectors: ComputedRef<any>[]): Writable<ComputedRef<any>>[];
}
export {};
